==> file_stream-0.3.1.tar: lib/file_stream/storage.ex <==
defmodule FileStream.Storage do
  @moduledoc """
  A behaviour for streaming files from storage.
  """

  @storage %{
    "gs" => FileStream.Storage.GCS,
    "http" => FileStream.Storage.HTTP,
    "https" => FileStream.Storage.HTTP,
    "file" => FileStream.Storage.Local,
    "sftp" => FileStream.Storage.SFTP
  }

  ################################
  # Types
  ################################

  @type t :: module()

  ################################
  # Callbacks
  ################################

  @doc """
  A callback to initialize a stream with the provided options.
  """
  @callback init(keyword()) :: any()

  @doc """
  A callback for `Collectable.into/1`.
  """
  @callback into(FileStream.t()) ::
              {FileStream.t(), (FileStream.t(), Collectable.command() -> FileStream.t())}

  @doc """
  A callback for `Enumerable.reduce/3`.
  """
  @callback reduce(FileStream.t(), Enumerable.acc(), Enumerable.reducer()) :: Enumerable.result()

  ################################
  # Public API
  ################################

  @doc """
  Similar to `from_uri/1` but raises a `FileStream.Error` if storage is not supported.
  """
  @spec from_uri!(URI.t()) :: t()
  def from_uri!(uri) do
    case from_uri(uri) do
      {:ok, storage} ->
        storage

      :error ->
        raise FileStream.Error, uri: uri, reason: "storage not supported"
    end
  end

  @doc """
  Determines the `FileStream.Storage` implementation based on a URI scheme.

  Returns `{:ok, storage}` on success or `:error` if storage is not supported.
  """
  @spec from_uri(URI.t()) :: {:ok, FileStream.Storage.t()} | :error
  def from_uri(uri) do
    custom_storage = Application.get_env(:file_stream, :storage, %{})
    storage = Map.merge(@storage, custom_storage)
    Map.fetch(storage, uri.scheme)
  end
end

==> file_stream-0.4.2.tar: lib/file_stream/storage.ex <==
defmodule FileStream.Storage do
  @moduledoc """
  A behaviour for streaming files from storage.
  """

  @storage %{
    "gs" => FileStream.Storage.GCS,
    "http" => FileStream.Storage.HTTP,
    "https" => FileStream.Storage.HTTP,
    "file" => FileStream.Storage.Local,
    "sftp" => FileStream.Storage.SFTP,
    "s3" => FileStream.Storage.S3
  }

  ################################
  # Types
  ################################

  @type t :: module()

  ################################
  # Callbacks
  ################################

  @doc """
  A callback to initialize a stream with the provided options.
  """
  @callback init(keyword()) :: any()

  @doc """
  A callback for `Collectable.into/1`.
  """
  @callback into(FileStream.t()) ::
              {FileStream.t(), (FileStream.t(), Collectable.command() -> FileStream.t())}

  @doc """
  A callback for `Enumerable.reduce/3`.
  """
  @callback reduce(FileStream.t(), Enumerable.acc(), Enumerable.reducer()) :: Enumerable.result()

  ################################
  # Public API
  ################################

  @doc """
  Similar to `from_uri/1` but raises a `FileStream.Error` if storage is not supported.
  """
  @spec from_uri!(URI.t()) :: t()
  def from_uri!(uri) do
    case from_uri(uri) do
      {:ok, storage} ->
        storage

      :error ->
        raise FileStream.Error, uri: uri, reason: "storage not supported"
    end
  end

  @doc """
  Determines the `FileStream.Storage` implementation based on a URI scheme.

  Returns `{:ok, storage}` on success or `:error` if storage is not supported.
  """
  @spec from_uri(URI.t()) :: {:ok, FileStream.Storage.t()} | :error
  def from_uri(uri) do
    custom_storage = Application.get_env(:file_stream, :storage, %{})
    storage = Map.merge(@storage, custom_storage)
    Map.fetch(storage, uri.scheme)
  end
end
